package eu.openaire.urls_controller;

import eu.openaire.urls_controller.services.FullTextsServiceImpl;
import eu.openaire.urls_controller.services.UrlsServiceImpl;
import eu.openaire.urls_controller.util.FileUtils;
import eu.openaire.urls_controller.util.UriBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.web.servlet.context.ServletWebServerApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.core.env.Environment;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.web.cors.CorsConfiguration;
import org.springframework.web.cors.CorsConfigurationSource;
import org.springframework.web.cors.UrlBasedCorsConfigurationSource;

import javax.annotation.PreDestroy;
import java.util.Arrays;
import java.util.Collections;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.TimeUnit;

@SpringBootApplication
@EnableScheduling
public class Application {

    private static final Logger logger = LoggerFactory.getLogger(Application.class);

    public static void main(String[] args) {
        SpringApplication.run(Application.class, args);
    }

    @Bean
    public CorsConfigurationSource corsConfigurationSource() {
        CorsConfiguration configuration = new CorsConfiguration();
        configuration.setAllowedOrigins(Collections.singletonList("*"));
        configuration.setAllowedMethods(Arrays.asList("GET", "POST", "PUT", "PATCH", "DELETE", "OPTIONS"));
        configuration.setAllowedHeaders(Arrays.asList("authorization", "content-type", "x-auth-token"));
        configuration.setExposedHeaders(Collections.singletonList("x-auth-token"));
        UrlBasedCorsConfigurationSource source = new UrlBasedCorsConfigurationSource();
        source.registerCorsConfiguration("/**", configuration);
        return source;
    }


    @PreDestroy
    public void preDestroy() {
        logger.info("Shutting down the threads..");

        shutdownThreads(UrlsServiceImpl.insertsExecutor);
        shutdownThreads(FileUtils.hashMatchingExecutor);
        shutdownThreads(FullTextsServiceImpl.backgroundExecutor);
        shutdownThreads(FullTextsServiceImpl.bulkImportExecutor);

        logger.info("Exiting..");
    }


    private void shutdownThreads(ExecutorService executorService)
    {
        executorService.shutdown();	// Define that no new tasks will be scheduled.
        try {
            if ( ! executorService.awaitTermination(1, TimeUnit.MINUTES) ) {
                logger.warn("The working threads did not finish on time! Stopping them immediately..");
                executorService.shutdownNow();
            }
        } catch (SecurityException se) {
            logger.error("Could not shutdown the threads in any way..!", se);
        } catch (InterruptedException ie) {
            try {
                executorService.shutdownNow();
            } catch (SecurityException se) {
                logger.error("Could not shutdown the threads in any way..!", se);
            }
        }
    }


    @Bean
    public CommandLineRunner setServerBaseUrl(Environment environment, ServletWebServerApplicationContext webServerAppCtxt)
    {
        return args -> new UriBuilder(environment, webServerAppCtxt);
    }

}